/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import jakarta.annotation.Nonnull;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;

@Component
class FachstelleModelAssembler
		implements RepresentationModelAssembler<Fachstelle, EntityModel<Fachstelle>> {

	@Override
	public EntityModel<Fachstelle> toModel(@Nonnull Fachstelle entity) {
		return ModelBuilder.fromEntity(entity)
				.addLink(linkTo(FachstelleController.class).slash(entity.getId()).withSelfRel())
				.buildModel();
	}

	@Override
	public CollectionModel<EntityModel<Fachstelle>> toCollectionModel(Iterable<? extends Fachstelle> entities) {
		return RepresentationModelAssembler.super.toCollectionModel(entities)
				.add(linkTo(FachstelleController.class).withSelfRel());
	}
}
