/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.user;

import java.util.Objects;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import io.grpc.Status;
import io.grpc.StatusRuntimeException;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
public class UserService {

	@Autowired(required = false)
	private UserRemoteService remoteService;

	public UserProfile getById(UserId userId) {
		try {
			return getUserRemoteService().getById(userId);
		} catch (StatusRuntimeException e) {
			if (isNotFound(e)) {
				// TODO use optional, rename function to 'find...', move errorhandling to
				// remoteService
				return null;
			}
			LOG.error("Error on loading UserProfile from UserManager.");
			throw new UserManagerUnavailableException();
		}
	}

	private boolean isNotFound(StatusRuntimeException e) {
		return e.getStatus().getCode().value() == Status.NOT_FOUND.getCode().value();
	}

	/** For package-internal use only */
	public Optional<UserId> getInternalId(UserId externalId) {
		try {
			return getUserRemoteService().getByExternalId(externalId).map(UserProfile::getId);
		} catch (UserManagerUnavailableException e) {
			return Optional.empty();
		}
	}

	private UserRemoteService getUserRemoteService() {
		if (Objects.nonNull(remoteService)) {
			return remoteService;
		}
		LOG.warn("UserManager is not configured.");
		throw new UserManagerUnavailableException();

	}
}