/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.hint;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;

@Service
class HintService {

	static final String USER_NO_ORGANISATIONS_EINHEIT_ID_CODE = "user.no.organisationsEinheitId";

	@Autowired
	private CurrentUserService currentUserService;

	public boolean hasHints() {
		return !findHints().isEmpty();
	}

	public List<Hint> findHints() {
		var hints = new ArrayList<Hint>();

		getHasUserNoOrganisationsEinheitIdHint().ifPresent(hints::add);

		return hints;
	}

	Optional<Hint> getHasUserNoOrganisationsEinheitIdHint() {
		if (getOrganisationsEinheitIds().isEmpty() && hasOrganisationsEinheitIdRequiredRole()) {
			return Optional.of(buildUserNoOrganisationsEinheitIdHint());
		}
		return Optional.empty();
	}

	private Hint buildUserNoOrganisationsEinheitIdHint() {
		return Hint.builder().code(USER_NO_ORGANISATIONS_EINHEIT_ID_CODE).build();
	}

	Collection<String> getOrganisationsEinheitIds() {
		return currentUserService.getUser().getOrganisationseinheitIds();
	}

	boolean hasOrganisationsEinheitIdRequiredRole() {
		return !currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER) && !currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE);
	}
}