/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class AktenzeichenChangeHistoryBuilder extends ChangeHistoryBuilder<AktenzeichenChangeHistoryBuilder> {

	static final String BODY_PROPERTY_AKTENZEICHEN = "aktenzeichen";

	public static AktenzeichenChangeHistoryBuilder builder() {
		return new AktenzeichenChangeHistoryBuilder();
	}

	@Override
	boolean isRelevant(Command command) {
		return command.getCommandOrder().equals(CommandOrder.SET_AKTENZEICHEN);
	}

	@Override
	CommandWithChangeValues toCommandWithChangeValues(CommandWithPrevious commandWithPrevious) {
		return new CommandWithChangeValues(
				commandWithPrevious.getCommand(),
				getAktenzeichenBeforeChange(commandWithPrevious),
				getAktenzeichenAfterChange(commandWithPrevious));
	}

	String getAktenzeichenBeforeChange(CommandWithPrevious commandWithPrevious) {
		return Optional.ofNullable(commandWithPrevious.getPrevious())
				.map(this::getAktenzeichen)
				.orElse(StringUtils.EMPTY);
	}

	String getAktenzeichenAfterChange(CommandWithPrevious commandWithPrevious) {
		return getAktenzeichen(commandWithPrevious.getCommand());
	}

	String getAktenzeichen(Command command) {
		return getValueFromCommandBody(BODY_PROPERTY_AKTENZEICHEN, command);
	}
}
