/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.downloadtoken;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.util.UUID;

import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

class DownloadTokenAuthenticationFilterTest {

	@InjectMocks
	private DownloadTokenAuthenticationFilter filter;
	@Mock
	private FilterChain filterChain;
	@Mock
	private HttpServletRequest request;
	@Mock
	private HttpServletResponse response;
	@Mock
	private DownloadTokenService downloadTokenService;

	@Test
	void shouldCallDoFilter() throws IOException, ServletException {
		doFilterInternal();

		verify(filterChain).doFilter(request, response);
	}

	@Nested
	class TestWithTokenParamter {

		final String TOKEN = UUID.randomUUID().toString();

		@BeforeEach
		void mockRequestParameter() {
			when(request.getParameter(anyString())).thenReturn(TOKEN);
		}

		@Test
		void shouldVerifyTokenIfPresent() throws Exception {
			doFilterInternal();

			verify(downloadTokenService).handleToken(request, TOKEN);
		}
	}

	private void doFilterInternal() throws ServletException, IOException {
		filter.doFilterInternal(request, response, filterChain);
	}
}