/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.GrpcVorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;

class KommentarRemoteServiceTest {

	@Spy
	@InjectMocks
	private KommentarRemoteService service;
	@Mock
	private VorgangAttachedItemServiceBlockingStub vorgangAttachedItemServiceStub;
	@Mock
	private KommentarMapper mapper;

	private GrpcVorgangAttachedItem kommentar = GrpcVorgangAttachedItemTestFactory.create();

	@DisplayName("Find by vorgangId")
	@Nested
	class TestFindByVorgangId {

		private GrpcFindVorgangAttachedItemRequest request = GrpcFindVorgangAttachedItemRequest.newBuilder()
				.setVorgangId(VorgangHeaderTestFactory.ID)
				.setItemName(KommentarRemoteService.ITEM_NAME)
				.build();
		private GrpcFindVorgangAttachedItemResponse response = GrpcFindVorgangAttachedItemResponse.newBuilder()
				.clearVorgangAttachedItems()
				.addVorgangAttachedItems(kommentar).build();

		@BeforeEach
		void mockStub() {
			when(vorgangAttachedItemServiceStub.find(any())).thenReturn(response);
		}

		@Test
		void shouldCallStub() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(vorgangAttachedItemServiceStub).find(request);
		}

		@Test
		void shouldCallMapper() {
			var result = service.findByVorgangId(VorgangHeaderTestFactory.ID);
			collectStreamElementsToTriggerLazyStream(result);

			verify(mapper).fromItem(any());
		}

		private void collectStreamElementsToTriggerLazyStream(Stream<Kommentar> stream) {
			stream.toList();
		}
	}

	@DisplayName("Find by id")
	@Nested
	class TestGetById {
		private GrpcVorgangAttachedItemResponse response = GrpcVorgangAttachedItemResponse.newBuilder()
				.setVorgangAttachedItem(GrpcVorgangAttachedItemTestFactory.create())
				.build();
		private GrpcVorgangAttachedItemRequest request = GrpcVorgangAttachedItemRequest.newBuilder()
				.setId(KommentarTestFactory.ID)
				.build();

		@BeforeEach
		void mockStub() {
			when(vorgangAttachedItemServiceStub.getById(any())).thenReturn(response);
			when(mapper.fromItem(any())).thenReturn(KommentarTestFactory.create());
		}

		@Test
		void shouldCallStub() {
			service.getById(KommentarTestFactory.ID);

			verify(vorgangAttachedItemServiceStub).getById(request);
		}

		@Test
		void shouldCallMapper() {
			service.getById(KommentarTestFactory.ID);

			verify(mapper).fromItem(any());
		}
	}
}
