/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class WiedervorlageControllerTest {

	private final String PATH = WiedervorlageController.WIEDERVORLAGE_PATH + "/";

	@Spy
	@InjectMocks
	private WiedervorlageController controller;
	@Mock
	private WiedervorlageService service;
	@Mock
	private WiedervorlageModelAssembler modelAssembler;
	@Mock
	private BinaryFileController binaryFileController;
	@Mock
	private WiedervorlageMapper mapper;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetById {

		private Wiedervorlage wiedervorlage = WiedervorlageTestFactory.create();

		@BeforeEach
		void mockService() {
			when(service.getById(any())).thenReturn(wiedervorlage);
		}

		@Test
		void shouldReturnStatusOk() throws Exception {
			callEndpoint().andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).getById(WiedervorlageTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toModel(wiedervorlage);
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(PATH + WiedervorlageTestFactory.ID));
		}
	}

	@Nested
	class TestFindByVorgang {

		private Wiedervorlage wiedervorlage = WiedervorlageTestFactory.create();

		@BeforeEach
		void mockService() {
			when(service.findByVorgangId(any())).thenReturn(Stream.of(wiedervorlage));
		}

		@Test
		void shouldReturnStatusOk() throws Exception {
			callEndpoint().andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toCollectionModel(ArgumentMatchers.<Stream<Wiedervorlage>>any(), eq(VorgangHeaderTestFactory.ID));
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(PATH).param(WiedervorlageController.PARAM_VORGANG_ID, VorgangHeaderTestFactory.ID));
		}
	}
}